@@ -35,4 +35,18 @@ def box_vertices(position_x, position_y, position_z, size_x, size_y, size_z, mat
     vertices[:,0] += position_x
     vertices[:,1] += position_y
     vertices[:,2] += position_z
-    return vertices
\ No newline at end of file
+    return vertices
+
+
+@numba.njit(cache=True)
+def vertices_inside_box(vertices, box_vertices):
+    """Returns indices of the vertices that are inside the box"""
+    P = box_vertices
+    u, v, w = P[1]-P[0], P[2]-P[0], P[6]-P[0]
+    pu, pv, pw = np.dot(vertices, u), np.dot(vertices, v), np.dot(vertices, w)
+    inside_box = np.where((pu <= np.dot(P[1], u)) & (pu >= np.dot(P[0], u))\
+                        & (pv <= np.dot(P[2], v)) & (pv >= np.dot(P[0], v))\
+                        & (pw <= np.dot(P[6], w)) & (pw >= np.dot(P[0], w))
+                        )[0]
+    return inside_box
+    
\ No newline at end of file
