@@ -34,6 +34,12 @@ class Point(Geometry):
         return Point(self.x - other.x, self.y - other.y, self.z - other.z)
 
 
+    def __mul__(self, factor:float) -> 'Point':
+        return Point(factor * self.x, factor * self.y, factor * self.z)
+    def __rmul__(self, factor:float) -> 'Point':
+        return self * factor
+
+
     def __repr__(self):
         return f'Point(x:{self.x:.2f}, y:{self.y:.2f}, z:{self.z:.2f})'
     
\ No newline at end of file
