@@ -1,9 +1,6 @@
-from openride.core.bounding_box import BoundingBox
 from openride.core.geometry import Geometry
-from openride.core.numba.polygon_utils import polygon_contains_point
 from openride.core.numba.transforms import transform_vertices
 from openride.core.point import Point
-from openride.core.polyline import Polyline
 
 import copy
 import numpy as np
@@ -46,20 +43,6 @@ class Polygon(Geometry):
     @classmethod
     def from_shapely(cls, polygon:geometry.Polygon) -> 'Polygon':
         return cls([Point(*c) for c in polygon.exterior.coords])
-
-
-    def contains(self, other:Geometry) -> bool:
-
-        if isinstance(other, Point):
-            return polygon_contains_point(self.vertices, other.x, other.y)
-
-        elif isinstance(other, BoundingBox):
-            return all([self.contains(Point(v[0],v[1])) for v in other.bird_eye_view_vertices])
-
-        elif isinstance(other, Polyline):
-            return all([self.contains(Point(v[0],v[1])) for v in other.vertices])
-
-        raise NotImplementedError
     
 
     def get_area(self) -> float:
