@@ -1,32 +1,46 @@
 from openride import Point
+from openride.core.numba.coordinate_systems import cartesian_to_spherical, spherical_to_cartesian
 
-from typing import Tuple
+from dataclasses import dataclass
+from enum import Enum
+from typing import Callable, Tuple, Union
 
 import numpy as np
 import vtk
 
 
 
-def spherical_to_cartesian(r, theta, phi) -> Tuple[float, float, float]:
-    x = r * np.sin(phi) * np.cos(theta)
-    y = r * np.sin(phi) * np.sin(theta)
-    z = r * np.cos(phi)
-    return x, y, z
+class Event(Enum):
+    MouseMoveEvent = 'MouseMoveEvent'
+    LeftButtonPressEvent = 'LeftButtonPressEvent'
+    MiddleButtonPressEvent = 'MiddleButtonPressEvent'
+    RightButtonPressEvent = 'RightButtonPressEvent'
+    LeftButtonReleaseEvent = 'LeftButtonReleaseEvent'
+    MiddleButtonReleaseEvent = 'MiddleButtonReleaseEvent'
+    RightButtonReleaseEvent = 'RightButtonReleaseEvent'
+    MouseWheelForwardEvent = 'MouseWheelForwardEvent'
+    MouseWheelBackwardEvent = 'MouseWheelBackwardEvent'
+    EnterEvent = 'EnterEvent'
+    LeaveEvent = 'LeaveEvent'
+    ExitEvent = 'ExitEvent'
+    KeyPressEvent = 'KeyPressEvent'
+    KeyReleaseEvent = 'KeyReleaseEvent'
 
 
-def cartesian_to_spherical(x, y, z) -> Tuple[float, float, float]:
-    r = (x**2 + y**2 + z**2)**0.5
-    theta = np.arctan(y / (x + np.finfo(float).eps))
-    if x < 0:
-        theta += np.pi
-    phi = np.arccos(z / (r + np.finfo(float).eps))
-    return r, theta, phi
+@dataclass
+class MouseEvent:
+    position:Tuple[int, int]
+    last_position:Tuple[int, int]
+
+@dataclass
+class KeyEvent:
+    key:str
 
 
 
 class Interactor:
 
-    def __init__(self, vtk_viewer):
+    def __init__(self, vtk_viewer, mouse_camera_interactions:bool=True):
 
         self.viewer = vtk_viewer
         
@@ -34,21 +48,25 @@ class Interactor:
 
         # Set an empty style, so this interactor does nothing when we call ProcessEvents() except our callbacks.
         self._interactor.SetInteractorStyle(vtk.vtkInteractorEventRecorder())
+        self._interactor.Initialize()
+
+        self._interactor.SetLightFollowCamera(True)
 
         # Callbacks
-        self._interactor.AddObserver('MouseMoveEvent',           self.__on_move, 1.0)
-        self._interactor.AddObserver('LeftButtonPressEvent',     self.__on_left_click, 1.0)
-        self._interactor.AddObserver('MiddleButtonPressEvent',   self.__on_middle_click, 1.0)
-        self._interactor.AddObserver('RightButtonPressEvent',    self.__on_right_click, 1.0)
-        self._interactor.AddObserver('LeftButtonReleaseEvent',   self.__on_left_release, 1.0)
-        self._interactor.AddObserver('MiddleButtonReleaseEvent', self.__on_middle_release, 1.0)
-        self._interactor.AddObserver('RightButtonReleaseEvent',  self.__on_right_release, 1.0)
-        self._interactor.AddObserver('MouseWheelForwardEvent',   self.__on_scroll_up, 1.0)
-        self._interactor.AddObserver('MouseWheelBackwardEvent',  self.__on_scroll_down, 1.0)
-        self._interactor.AddObserver('EnterEvent',               self.__on_enter, 1.0)
-        self._interactor.AddObserver('LeaveEvent',               self.__on_leave, 1.0)
-        self._interactor.AddObserver('ExitEvent',                self.__on_exit, 1.0)
-        self._interactor.Initialize()
+        self.set_callback(Event.MouseMoveEvent,           self.__on_move)
+        self.set_callback(Event.LeftButtonPressEvent,     self.__on_left_click)
+        self.set_callback(Event.MiddleButtonPressEvent,   self.__on_middle_click)
+        self.set_callback(Event.RightButtonPressEvent,    self.__on_right_click)
+        self.set_callback(Event.LeftButtonReleaseEvent,   self.__on_left_release)
+        self.set_callback(Event.MiddleButtonReleaseEvent, self.__on_middle_release)
+        self.set_callback(Event.RightButtonReleaseEvent,  self.__on_right_release)
+        self.set_callback(Event.MouseWheelForwardEvent,   self.__on_scroll_up)
+        self.set_callback(Event.MouseWheelBackwardEvent,  self.__on_scroll_down)
+        self.set_callback(Event.EnterEvent,               self.__on_enter)
+        self.set_callback(Event.LeaveEvent,               self.__on_leave)
+        self.set_callback(Event.ExitEvent,                self.__on_exit)
+
+        self.set_mouse_camera_interactions(mouse_camera_interactions)
 
         self._point_placer = vtk.vtkFocalPlanePointPlacer()
 
@@ -58,12 +76,31 @@ class Interactor:
         self.middle_mouse = False
         self.right_mouse = False
 
+
     def toggle(self, state:bool):
         if state: self._interactor.Enable()
         else: self._interactor.Disable()
 
+
+    def set_mouse_camera_interactions(self, state:bool):
+        self._mouse_camera_interactions = state
+
+
     def update(self):
-        self._interactor.ProcessEvents()        
+        self._interactor.ProcessEvents()
+
+
+    def set_callback(self, event:Event, callback:Callable[[Union[MouseEvent, KeyEvent]], None]):
+
+        if event is Event.KeyPressEvent or event is Event.KeyReleaseEvent:
+            def _callback(key, _):
+                callback(KeyEvent(key.GetKeySym()))
+        else:
+            def _callback(mouse, _):
+                callback(MouseEvent(mouse.GetEventPosition(), mouse.GetLastEventPosition()))
+
+        self._interactor.AddObserver(event.value, _callback, 1.0)
+
 
     def __on_enter(self, *args):
         self.cursor_is_in_window = True
@@ -90,9 +127,11 @@ class Interactor:
     def __on_exit(self, *args):
         exit()
 
-    def __on_move(self, event, _):
+
+    def __on_move(self, mouse_event:MouseEvent):
         if not self.cursor_is_in_window: return
-        self.cursor = event.GetEventPosition()
+        if not self._mouse_camera_interactions: return
+        self.cursor = mouse_event.position
 
         position = self.viewer.camera.get_position()
         focus    = self.viewer.camera.get_focus()
@@ -104,7 +143,7 @@ class Interactor:
             focus -= delta
 
         if self.middle_mouse:
-            x0, y0 = event.GetLastEventPosition()
+            x0, y0 = mouse_event.last_position
             d, horizontal_angle, vertical_angle = self.__get_camera_spherical_coordinates()
             factor = (d / 1400)
             delta_y = self.cursor[1] - y0
@@ -112,7 +151,7 @@ class Interactor:
             focus.z -= factor * delta_y
 
         if self.right_mouse:
-            x0, y0 = event.GetLastEventPosition()
+            x0, y0 = mouse_event.last_position
             delta_x = self.cursor[0] - x0
             delta_y = self.cursor[1] - y0
             
